@@ -13,7 +13,7 @@
 ##############################################################################
 """Test tcpdoc
 
-$Id: tests.py 116533 2010-09-17 15:47:35Z icemac $
+$Id: tests.py 119446 2011-01-07 19:50:06Z mgedmin $
 """
 from doctest import DocTestSuite
 import os
@@ -31,7 +31,8 @@ from zope.app.publication.requestpublicationfactories import BrowserFactory
 from zope.app.testing import functional
 from zope.app.testing.dochttp import dochttp
 import transaction
-from zope.app.testing.functional import SampleFunctionalTest, BrowserTestCase
+from zope.app.testing.functional import SampleFunctionalTest
+from zope.app.testing.functional import BrowserTestCase, HTTPTestCase
 from zope.app.testing.functional import FunctionalDocFileSuite
 from zope.app.testing.functional import FunctionalTestCase
 from zope.app.testing.functional import FunctionalTestSetup
@@ -40,7 +41,6 @@ from zope.app.testing.testing import AppTestingLayer
 from zope.app.testing.testing import FailingKlass
 
 
-
 HEADERS = """\
 HTTP/1.1 200 OK
 Content-Type: text/plain
@@ -219,9 +219,9 @@ class CookieHandlerTestCase(unittest.TestCase):
         self.handler.saveCookies(response)
         self.assertEqual(len(self.handler.cookies), 2)
         self.assert_(self.handler.cookies['spam'].OutputString() in
-                         ('spam=eggs; Path=/foo;','spam=eggs; Path=/foo'))
+                         ('spam=eggs; Path=/foo;', 'spam=eggs; Path=/foo'))
         self.assert_(self.handler.cookies['monty'].OutputString() in
-                         ('monty=python;','monty=python'))
+                         ('monty=python;', 'monty=python'))
 
     def test_httpCookie(self):
         cookies = self.handler.cookies
@@ -246,6 +246,41 @@ class CookieHandlerTestCase(unittest.TestCase):
     # standard library has tests for (we hope).
 
 
+class HTTPFunctionalTest(HTTPTestCase):
+
+    def testNoDefaultReferer(self):
+        # There should be no referer set in the request by default.
+        r = self.makeRequest()
+        self.assertRaises(KeyError, r.environment.__getitem__, 'HTTP_REFERER')
+
+
+class BrowserFunctionalTest(BrowserTestCase):
+
+    def testNoDefaultReferer(self):
+        # There should be no referer set in the request by default.
+        r = self.makeRequest()
+        self.assertRaises(KeyError, r.environment.__getitem__, 'HTTP_REFERER')
+
+
+class HTTPCallerFunctionalTest(FunctionalTestCase):
+
+    def testNoDefaultReferer(self):
+        # There should be no referer set in the request by default.
+        from zope.app.testing.functional import HTTPCaller
+        http = HTTPCaller()
+        response = http("GET /++skin++Basic HTTP/1.1\n\n")
+        self.assertRaises(KeyError, response._request.environment.__getitem__,
+                          'HTTP_REFERER')
+
+    def testRemoteAddr(self):
+        # There should be a REMOTE_ADDR in the request by default.
+        from zope.app.testing.functional import HTTPCaller
+        http = HTTPCaller()
+        response = http("GET / HTTP/1.1\n\n")
+        self.assertEqual(response._request.environment['REMOTE_ADDR'],
+                         '127.0.0.1')
+
+
 class GetCookies(object):
     """Get all cookies set."""
 
@@ -281,8 +316,7 @@ class CookieFunctionalTest(BrowserTestCase):
         super(CookieFunctionalTest, self).setUp()
         self.assertEqual(
                 len(self.cookies.keys()), 0,
-                'cookies store should be empty'
-                )
+                'cookies store should be empty')
 
         zope.configuration.xmlconfig.string(r'''
         <configure xmlns="http://namespaces.zope.org/browser">
@@ -324,16 +358,16 @@ class CookieFunctionalTest(BrowserTestCase):
         response = self.publish('/')
 
         self.assertEquals(response.getStatus(), 200)
-        self.assert_(not response._request._cookies.has_key('aid'))
+        self.assert_('aid' not in response._request._cookies)
         self.assertEquals(response._request._cookies['bid'], 'bval')
 
     def testHttpCookieHeader(self):
         # Passing an HTTP_COOKIE header to publish adds cookies
         response = self.publish('/', env={
-            'HTTP_COOKIE': '$Version=1, aid=aval; $Path=/sub/folder, bid=bval'
-            })
+            'HTTP_COOKIE':
+                '$Version=1, aid=aval; $Path=/sub/folder, bid=bval'})
         self.assertEquals(response.getStatus(), 200)
-        self.failIf(response._request._cookies.has_key('aid'))
+        self.failIf('aid' in response._request._cookies)
         self.assertEquals(response._request._cookies['bid'], 'bval')
 
     def testStickyCookies(self):
@@ -363,6 +397,7 @@ class SkinsAndHTTPCaller(FunctionalTestCase):
         response = http("GET /++skin++Basic HTTP/1.1\n\n")
         self.assert_("zopetopBasic.css" in str(response))
 
+
 class RetryProblemFunctional(FunctionalTestCase):
 
     def setUp(self):
@@ -391,6 +426,7 @@ Authorization: Basic mgr:mgrpw
         self.assertNotEqual(response.getStatus(), 599)
         self.assertEqual(response.getStatus(), 500)
 
+
 class RetryProblemBrowser(BrowserTestCase):
     def setUp(self):
         super(RetryProblemBrowser, self).setUp()
@@ -415,6 +451,7 @@ class RetryProblemBrowser(BrowserTestCase):
 
 ftesting_zcml = os.path.join(here, 'ftesting.zcml')
 
+
 def doctest_FunctionalTestSetup_clears_global_utilities():
     """Test that FunctionalTestSetup doesn't leave global utilities.
 
@@ -455,6 +492,7 @@ def doctest_FunctionalTestSetup_clears_global_utilities():
 
 empty_zcml = os.path.join(here, 'empty.zcml')
 
+
 def doctest_FunctionalTestSetup_supports_product_config():
     """Test that FunctionalTestSetup configures products.
 
@@ -609,13 +647,15 @@ def doctest_ZCMLLayer_carries_product_configuration():
 
 def test_suite():
     checker = RENormalizing([
-        (re.compile(r'^HTTP/1.1 (\d{3}) .*?\n'), 'HTTP/1.1 \\1\n')
-        ])
+        (re.compile(r'^HTTP/1.1 (\d{3}) .*?\n'), 'HTTP/1.1 \\1\n')])
     SampleFunctionalTest.layer = AppTestingLayer
     CookieFunctionalTest.layer = AppTestingLayer
     SkinsAndHTTPCaller.layer = AppTestingLayer
     RetryProblemFunctional.layer = AppTestingLayer
     RetryProblemBrowser.layer = AppTestingLayer
+    HTTPFunctionalTest.layer = AppTestingLayer
+    BrowserFunctionalTest.layer = AppTestingLayer
+    HTTPCallerFunctionalTest.layer = AppTestingLayer
 
     doc_test = FunctionalDocFileSuite('doctest.txt', 'cookieTestOne.txt',
         'cookieTestTwo.txt', checker=checker)
@@ -628,6 +668,9 @@ def test_suite():
         unittest.makeSuite(CookieHandlerTestCase),
         DocTestSuite(),
         unittest.makeSuite(SampleFunctionalTest),
+        unittest.makeSuite(HTTPFunctionalTest),
+        unittest.makeSuite(BrowserFunctionalTest),
+        unittest.makeSuite(HTTPCallerFunctionalTest),
         unittest.makeSuite(CookieFunctionalTest),
         unittest.makeSuite(SkinsAndHTTPCaller),
         unittest.makeSuite(RetryProblemFunctional),
